package ru.vyatsu.qr_access_auth_server

import org.springframework.security.oauth2.core.AuthorizationGrantType
import org.springframework.security.oauth2.core.ClientAuthenticationMethod
import org.springframework.security.oauth2.server.authorization.client.JdbcRegisteredClientRepository.RegisteredClientRowMapper
import org.springframework.security.oauth2.server.authorization.client.RegisteredClient
import org.springframework.util.StringUtils
import java.sql.ResultSet

class KotlinRegisteredClientRowMapper : RegisteredClientRowMapper() {

    override fun mapRow(rs: ResultSet, rowNum: Int): RegisteredClient? {
        val clientIdIssuedAt = rs.getTimestamp("client_id_issued_at")
        val clientSecretExpiresAt = rs.getTimestamp("client_secret_expires_at")
        val clientAuthenticationMethods =
            StringUtils.commaDelimitedListToSet(rs.getString("client_authentication_methods"))
        val authorizationGrantTypes = StringUtils.commaDelimitedListToSet(rs.getString("authorization_grant_types"))
        val redirectUris = StringUtils.commaDelimitedListToSet(rs.getString("redirect_uris"))
        val postLogoutRedirectUris = StringUtils.commaDelimitedListToSet(rs.getString("post_logout_redirect_uris"))
        val clientScopes = StringUtils.commaDelimitedListToSet(rs.getString("scopes"))
        val builder = RegisteredClient.withId(rs.getString("id"))
            .clientId(rs.getString("client_id"))
            .clientIdIssuedAt(clientIdIssuedAt?.toInstant())
            .clientSecret(rs.getString("client_secret"))
            .clientSecretExpiresAt(clientSecretExpiresAt?.toInstant())
            .clientName(rs.getString("client_name"))
            .clientAuthenticationMethods { authenticationMethods ->
                clientAuthenticationMethods.forEach { authenticationMethod ->
                    authenticationMethods.add(resolveClientAuthenticationMethod(authenticationMethod))
                }
            }
            .authorizationGrantTypes { grantTypes ->
                authorizationGrantTypes.forEach { grantType ->
                    grantTypes.add(resolveAuthorizationGrantType(grantType))
                }
            }
            .redirectUris { uris -> uris.addAll(redirectUris) }
            .postLogoutRedirectUris { uris ->
                uris.addAll(postLogoutRedirectUris)
            }
            .scopes { scopes -> scopes.addAll(clientScopes) }
        return builder.build()
    }

    private fun resolveAuthorizationGrantType(authorizationGrantType: String): AuthorizationGrantType {
        return if (AuthorizationGrantType.AUTHORIZATION_CODE.value == authorizationGrantType) {
            AuthorizationGrantType.AUTHORIZATION_CODE
        } else if (AuthorizationGrantType.CLIENT_CREDENTIALS.value == authorizationGrantType) {
            AuthorizationGrantType.CLIENT_CREDENTIALS
        } else {
            if (AuthorizationGrantType.REFRESH_TOKEN.value == authorizationGrantType) AuthorizationGrantType.REFRESH_TOKEN
            else AuthorizationGrantType(authorizationGrantType)
        }
    }

    private fun resolveClientAuthenticationMethod(clientAuthenticationMethod: String): ClientAuthenticationMethod {
        return if (ClientAuthenticationMethod.CLIENT_SECRET_BASIC.value == clientAuthenticationMethod) {
            ClientAuthenticationMethod.CLIENT_SECRET_BASIC
        } else if (ClientAuthenticationMethod.CLIENT_SECRET_POST.value == clientAuthenticationMethod) {
            ClientAuthenticationMethod.CLIENT_SECRET_POST
        } else {
            if (ClientAuthenticationMethod.NONE.value == clientAuthenticationMethod) ClientAuthenticationMethod.NONE
            else ClientAuthenticationMethod(clientAuthenticationMethod)
        }
    }
}